﻿using System;

using Thread = System.Threading.Thread;
using Dispatcher = System.Windows.Threading.Dispatcher;
using System.IO;

namespace OptimisationManagerExtention
{
    /// <summary>
    /// Класс для связки графического интерфейса с метатрейдером
    /// </summary>
    public class MQL5Connector
    {
        /// <summary>
        /// Поле хранящее в себе указатель на запущенный графический интерфейс
        /// </summary>
        private static View.ExtentionGUI instance;
        /// <summary>
        /// Метод осуществляющий запуск графического интерфейса. 
        /// Причем запуск осуществляется лишь одного интерфейса из одного робота. 
        /// т.е. при запуске осуществляется проверка были ли ранее запущен графический интерфейс, 
        /// если да - то запуск нового отклоняется
        /// </summary>
        /// <param name="pathToTerminal">Путь к изменяемой папке терминалла</param>
        public static void Instance(string terminalID)
        {
            // проверка был ли ранее запущен графический интерфейс
            if (instance == null)
            {
                // Переменная вторичного потока - потока графического интерфейса (графика запускается во вторичном потоке)
                // и ее инстанцирование с передачей лямбда выражения описываюзщего порядок запуска графики
                Thread t = new Thread(() =>
                {
                    // Инстанцирование класса графического интерфейса и его отображение (запуск графики)
                    instance = new View.ExtentionGUI();
                    instance.Show();
                    // Подписка на событие закрытия окна графики - если окно закрыто то 
                    // полю где зранилась ссылка на графический интерфейс присваивается значение null
                    instance.Closed += (object o, EventArgs e) => { instance = null; };

                    // Запуск Диспатчера потока графического интерфейса
                    Dispatcher.Run();
                });


                MainTerminalID = terminalID;

                // Запуск вторичного потока
                t.SetApartmentState(System.Threading.ApartmentState.STA);
                t.Start();
            }
        }
        /// <summary>
        /// МПолучает информацию активно ли окно
        /// </summary>
        /// <returns>true если активно и false если закрыто</returns>
        public static bool IsWindowActive() => instance != null;
        /// <summary>
        /// Main Terminal ID
        /// </summary>
        internal static string MainTerminalID { get; private set; }
        internal static Dispatcher CurrentDispatcher => ((instance == null) ? Dispatcher.CurrentDispatcher : instance.Dispatcher);
    }
}
