﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Collections.ObjectModel;
using System;
using OptimisationManagerExtention.Model.Terminal;

namespace OptimisationManagerExtention.Model
{
    /// <summary>
    /// Интерфейс модели
    /// </summary>
    interface IExtentionGUI_M : INotifyPropertyChanged
    {
        #region Properties

        /// <summary>
        /// Устанавливает когда доступна для редактирования таблица с выбранными терминаллами а когда нет
        /// </summary>
        bool IsTerminalsLVEnabled { get; }

        /// <summary>
        /// Список параметров робота как в таблице перед началом оптимизации
        /// </summary>
        List<FileReaders.ParamsItem> BotParams { get; }

        /// <summary>
        /// Статус выполняемой операции
        /// </summary>
        VarKeeper<string> Status { get; }
        /// <summary>
        /// Значение прогресс бара
        /// </summary>
        VarKeeper<double> PB_Value { get; }

        /// <summary>
        /// ID терминалов на стартавой странице
        /// определяются при старте программы и неизменны до ее завершения 
        /// </summary>
        ObservableCollection<string> TerminalsID { get; }

        /// <summary>
        /// Таблица с результатами оптимизации на оптимизируемом отрезке времени
        /// </summary>
        DataTable HistoryOptimisationResults { get; }
        /// <summary>
        /// Таблица с результатами оптимизаций на форвардном отрезке времени
        /// </summary>
        DataTable ForvardOptimisationResults { get; }
        /// <summary>
        /// Коллекция с заголовками отображаемых колонок в таблицах с результатами оптимизаций
        /// </summary>
        ObservableCollection<ViewExtention.ColumnDescriptor> OptimisationResultsColumnHeadders { get; }
        /// <summary>
        /// Список ID терминаллов на которых производилась оптимизация
        /// </summary>
        ObservableCollection<string> TerminalsAfterOptimisation { get; }
        /// <summary>
        /// индекс выбранного терминалла для конкретного робота
        /// </summary>
        VarKeeper<int> TerminalsAfterOptimisation_Selected { get; set; }

        /// <summary>
        /// Список роботов прошедших оптимизацию - роботы в списке не повторяются, а вот терминаллов на одного робота может быть несколько
        /// </summary>
        ObservableCollection<string> BotsAfterOptimisation { get; }
        /// <summary>
        /// индекс выбранного прооптимизированного робота
        /// </summary>
        VarKeeper<int> BotsAfterOptimisation_Selected { get; set; }
        #endregion

        /// <summary>
        /// Метод загружающий историю оптимизаций в таблицы
        /// [History/Forvard]OptimisationResults
        /// </summary>
        /// <param name="terminal_ind">Индекс выбранного робота из списка "BotList"</param>
        /// <param name="bot_ind">Индекс выбранного робота из списка "BotList"</param>
        void LoadOptimisations();
        /// <summary>
        /// Метод проверяющий есть ли сейчас возможность загрузки отчета оптимизации или же нет
        /// </summary>
        /// <returns>true если есть возможность, false если нет</returns>
        //bool IsEnableLoadOptimisationReport();
        /// <summary>
        /// Данный метод вызывается при выборе эксперта для оптимизации / теста 
        /// из таблицы с терминаллами на вкладке Settings
        /// 
        /// Во время его работы :
        /// 1. Сохранятся в прошлом выбранном роботе его оптимизационные настройки - "optimisationInputData"
        /// 2. Загружаются параметры нововыбранного робота в таблицу "BotParams"
        /// 3. в переменную "optimisationInputData" - передается значения настроек оптимизации 
        ///    для нововыбранного робота
        /// </summary>
        /// <param name="fullExpertName">Путь к эксперту относительно папки ~/Experts</param>
        /// <param name="TerminalID">ID запускаемого терминалла</param>
        /// <param name="optimisationInputData">настройки оптимизации</param>
        void LoadBotParams(string fullExpertName,
            string TerminalID,
            out OptimisationInputData? optimisationInputData);
        /// <summary>
        /// Метод получает полные имена роботов для выбранного терминалла относительно папки ~/Experts
        /// </summary>
        /// <param name="terminalIndex">Индекс выбранного терминалла</param>
        /// <returns>
        /// Список полных имен роботов(с учетом пути относительно папки ~/Experts)
        /// </returns>
        List<string> GetBotNamesList(int terminalIndex);
        /// <summary>
        /// Метод получает список доступных активов из выбранного терминала
        /// </summary>
        /// <param name="terminalIndex">индекс выбранного терминалла</param>
        /// <returns>Список доступных активов</returns>
       // List<string> GetAssetNames(int terminalIndex);
        /// <summary>
        /// Метод получает логин из мнмциализационного файла для выбранного терминалла
        /// </summary>
        /// <param name="terminalIndex">индекс выбранного терминалла</param>
        /// <returns>Логин терминалла</returns>
        uint? GetCurrentLogin(int terminalIndex);
        /// <summary>
        /// Проверка перед запуском оптимизации или же теста
        /// </summary>
        /// <param name="SelectedTerminals">список выбранных терминалов для оптимизации/теста</param>
        /// <returns>результат проверки</returns>
        //bool IsEnableToStartOptimisationOrTest(List<ViewModel.TerminalAndBotItem> SelectedTerminals);
        /// <summary>
        /// Метод запускающий тест или же оптимизацию
        /// </summary>
        /// <param name="SelectedTerminals">список выбранных терминалов для оптимизации/теста</param>
        void StartOptimisationOrTest(List<ViewModel.TerminalAndBotItem> SelectedTerminals);
        /// <summary>
        /// Запуск теста по событию двойного клика
        /// </summary>
        /// <param name="TableType">Тип таблицы</param>
        /// <param name="rowIndex">Индекс выбранной натписи</param>
        /// <param name="terminalData">Параметры для входа в терминалл и путь к роботу</param>
        void StartTest(ENUM_TableType TableType, int rowIndex);
        /// <summary>
        /// Вызывается при удалении переданного терминалла из списка терминаллов
        /// В методе удаляются параметры для выбранного робота в данном терминале
        /// </summary>
        /// <param name="item">выбранный для удаления терминалл</param>
        bool RemoveBotParams(string TerminalID);
        /// <summary>
        /// Проверяем можно ли добавить новый терминалл. Нельзя добавлять если в текущий попнт идет оптимизация
        /// </summary>
        /// <returns></returns>
        bool IsEnableToAddNewTerminal();
        void SelectNewBotsAfterOptimisation_forNewTerminal();
        void UpdateTerminalOptimisationsParams(OptimisationInputData optimisationInputData);
        // string GetPathToTerminalFolder();
    }

    #region Accessory objects 

    /// <summary>
    /// Enum характеризующий тип таблиц с результатами оптимизаций
    /// </summary>
    enum ENUM_TableType
    {
        History,
        Forvard
    }

    /// <summary>
    /// Структыра оптимизационных настроек робота
    /// </summary>
    struct OptimisationInputData
    {
        public void Copy(OptimisationInputData data)
        {
            Login = data.Login;
            ForvardDate = data.ForvardDate;
            IsVisual = data.IsVisual;
            Deposit = data.Deposit;
            Laverage = data.Laverage;
            Currency = data.Currency;
            DepositIndex = data.DepositIndex;
            ExecutionDelayIndex = data.ExecutionDelayIndex;
            ModelIndex = data.ModelIndex;
            CurrencyIndex = data.CurrencyIndex;
            LaverageIndex = data.LaverageIndex;
            OptimisationCriteriaIndex = data.OptimisationCriteriaIndex;
        }
        /// <summary>
        /// Логин видимый роботом в тестере / оптимизаторе
        /// </summary>
        public uint? Login;
        /// <summary>
        /// Дата начала форвардного периода
        /// </summary>
        public DateTime ForvardDate;
        /// <summary>
        /// Признак старта оптимизации в визуальном режиме
        /// </summary>
        public bool IsVisual;
        /// <summary>
        /// Депозит с которым стартует оптимизация
        /// </summary>
        public int Deposit;
        /// <summary>
        /// Кредитное плече
        /// </summary>
        public string Laverage;
        /// <summary>
        /// Валюта в которой измеряется прибыль
        /// </summary>
        public string Currency;
        /// <summary>
        /// Набор индексов для параметров оптимизации
        /// </summary>
        public int DepositIndex, ExecutionDelayIndex, ModelIndex,
                   CurrencyIndex, LaverageIndex, OptimisationCriteriaIndex;
        /// <summary>
        /// Виды подачи графика в робота при тестировании / оптимизации
        /// </summary>
        public ENUM_Model Model => GetEnum<ENUM_Model>(ModelIndex);
        /// <summary>
        /// Критерии оптимизационного отбора при оптимизации
        /// </summary>
        public ENUM_OptimisationCriteria GetOptimisationCriteria => GetEnum<ENUM_OptimisationCriteria>(OptimisationCriteriaIndex);
        /// <summary>
        /// Виды эмуляции задержек при исполнении ордера
        /// </summary>
        public ENUM_ExecutionDelay ExecutionDelay => GetEnum<ENUM_ExecutionDelay>(ExecutionDelayIndex);
        /// <summary>
        /// Преобразование индекса из массива имен перечисления в соответствующее перечисление
        /// </summary>
        /// <typeparam name="T">Тип перечисления к которому стоит привести индекс</typeparam>
        /// <param name="ind">переданный индекс</param>
        /// <returns>Элемент запрашиваемого перечисления</returns>
        private T GetEnum<T>(int ind)
        {
            Type type = typeof(T);
            string[] arr = Enum.GetNames(type);
            return (T)Enum.Parse(type, arr[ind]);
        }
    }

    /// <summary>
    /// Фабрика по подстановки модели в графический интерфейс
    /// </summary>
    class ModelCreator
    {
        /// <summary>
        /// Модель
        /// </summary>
        private static IExtentionGUI_M testModel;

        /// <summary>
        /// Property возвращающее либо модель (если она не была подменена) либо побмененную (для тестов) модель
        /// </summary>
        internal static IExtentionGUI_M Model => testModel ?? null;

        /// <summary>
        /// Метод подмены модели. подставляет тестовую модель что бы можно было отдельно тестировать графику от логики
        /// </summary>
        /// <param name="model">тестовая модель - подставляется извне</param>
        [System.Diagnostics.Conditional("DEBUG")]
        public static void SetModel(IExtentionGUI_M model)
        {
            testModel = model;
        }
    }

    #endregion
}
