﻿using System;
using System.Runtime.InteropServices;
using System.Text;

using File = System.IO.File;

namespace OptimisationManagerExtention.Model.Terminal
{

    #region Accessory objects
    /// <summary>
    /// Вид секции
    /// </summary>
    enum ENUM_SectionType
    {
        Common,
        Charts,
        Experts,
        Objects,
        Email,
        StartUp,
        Tester
    }

    /// <summary>
    /// Тип форвардного тестирования
    /// </summary>
    enum ENUM_ForvardMode
    {
        Disabled,
        Half, //1/2
        Third, // 1/3
        Fourth, // 1/4
        Custom
    }

    /// <summary>
    /// Виды эмуляции задержки исполнения ордера
    /// </summary>
    enum ENUM_ExecutionDelay
    {
        Without_Delay = 0,
        Random_Delay = -1,
        ms_2 = 2,
        ms_1 = 1,
        ms_5 = 5,
        ms_10 = 10,
        ms_20 = 20,
        ms_50 = 50,
        ms_100 = 100,
        ms_500 = 500,
        ms_1000 = 1000
    }

    /// <summary>
    /// Виды способа подачи графика в робот при тестировании / оптимизации
    /// </summary>
    enum ENUM_Model
    {
        Every_tick,
        OHLC_1_minute,
        Open_pricies_only,
        Math_calculations,
        Every_tick_based_on_real_ticks
    }

    /// <summary>
    /// Критерии оптимизационного отбора при оптимизации
    /// </summary>
    enum ENUM_OptimisationCriteria
    {
        Balance,
        Balance__Profit_factor,
        Balance__Expected_Payoff,
        Balance__Min_DD,
        Balance__Recovery_factor,
        Balance__Sharp_ratio,
        Custom
    }

    /// <summary>
    /// Таймфреймы
    /// </summary>
    enum ENUM_Timeframes
    {
        M1 = 1,
        M2 = 2,
        M3 = 3,
        M4 = 4,
        M5 = 5,
        M6 = 6,
        M10 = 10,
        M12 = 12,
        M15 = 15,
        M20 = 20,
        M30 = 30,
        H1 = 16385,
        H2 = 16386,
        H3 = 16387,
        H4 = 16388,
        H6 = 16390,
        H8 = 16392,
        H12 = 16396,
        D1 = 16408,
        W1 = 32769,
        MN1 = 49153
    }

    /// <summary>
    /// Способы оптимизации робота
    /// </summary>
    enum ENUM_OptimisationMode
    {
        Disabled,
        Slow_complete_algorithm,
        Fast_genetic_based_algorithm,
        All_symbols_selected_in_Market_Watch
    }

    /// <summary>
    /// Тип прокси сервера
    /// </summary>
    enum ENUM_ProxyType
    {
        SOCKS4,
        SOCKS5,
        HTTP
    }

    /// <summary>
    /// IPv4 адрес сервера и порт
    /// </summary>
    class ServerAddressKeeper
    {
        public ServerAddressKeeper(IPv4Adress ip, uint port)
        {
            IP = ip;
            Port = port;
        }
        public ServerAddressKeeper(string adress)
        {
            if (string.IsNullOrEmpty(adress) || string.IsNullOrWhiteSpace(adress))
                throw new ArgumentException("adress is incorrect");

            string[] data = adress.Split(':');

            if (data.Length != 2)
                throw new ArgumentException("adress is incorrect");

            IP = new IPv4Adress(data[0]);
            Port = Convert.ToUInt32(data[1]);
        }

        public IPv4Adress IP { get; }
        public uint Port { get; }

        public string Address => $"{IP.ToString()}:{Port}";
    }

    /// <summary>
    /// IPv4 адрес сервера
    /// </summary>
    struct IPv4Adress
    {
        public IPv4Adress(string adress)
        {
            string[] ip = adress.Split('.');
            if (ip.Length != 4)
                throw new ArgumentException("ip is incorrect");

            part_1 = (char)Convert.ToInt32(ip[0]);
            part_2 = (char)Convert.ToInt32(ip[1]);
            part_3 = (char)Convert.ToInt32(ip[2]);
            part_4 = (char)Convert.ToInt32(ip[3]);
        }

        public char part_1;
        public char part_2;
        public char part_3;
        public char part_4;

        public new string ToString()
        {
            return $"{(int)part_1}.{(int)part_2}.{(int)part_3}.{(int)part_4}";
        }
    }
    #endregion
}
