﻿using System;

namespace OptimisationManagerExtention.Model
{
    /// <summary>
    /// Класс - хранящий в себе переменную _Var типа T_keeper.
    /// Можем неявно приводиться к типу T_keeper а так же менять значение хранимой переменной
    /// В момент изменения значения - уведомляет об этом всех подписавшихся
    /// </summary>
    /// <typeparam name="T_keeper">Тип хранимой переменной</typeparam>
    class VarKeeper<T_keeper>
    {
        /// <summary>
        /// Конструктор задающи идентификационное имя переменной
        /// </summary>
        /// <param name="propertyName">идентификационное имя переменной</param>
        public VarKeeper(string propertyName)
        {
            this.propertyName = propertyName;
        }
        /// <summary>
        /// Конструктор задающий идентификационное имя переменной 
        /// и первоночальное значение переменной
        /// </summary>
        /// <param name="PropertyName">идентификационное имя переменной</param>
        /// <param name="Var">первоночальное значение переменной</param>
        public VarKeeper(string PropertyName, T_keeper Var) : this(PropertyName)
        {
            _Var = Var;
        }
        /// <summary>
        /// Перегрузка оператора неявного приведения типов.
        /// Переводит данный тип к T_keeper
        /// </summary>
        /// <param name="obj"></param>
        public static implicit operator T_keeper(VarKeeper<T_keeper> obj)
        {
            return obj._Var;
        }
        /// <summary>
        /// хранимая переменная 
        /// </summary>
        protected T_keeper _Var;
        /// <summary>
        /// Идентификационное имя переменной
        /// </summary>
        public readonly string propertyName;

        #region Event 
        /// <summary>
        /// Событие уведомляющее об изменении хранимой переменной
        /// </summary>
        public event Action<string> PropertyChanged;
        /// <summary>
        /// Метод вызывающий событие уведомляющее об изменении хранимой переменной
        /// </summary>
        protected void OnPropertyChanged()
        {
            PropertyChanged?.Invoke(propertyName);
        }
        #endregion
        /// <summary>
        /// Метод устанавливающий значение переменной значением value
        /// </summary>
        /// <param name="value">новое значение переменной</param>
        public void SetVar(T_keeper value)
        {
            SetVarSilently(value);
            OnPropertyChanged();
        }
        public void SetVarSilently(T_keeper value)
        {
            _Var = value;
        }
    }
}
