﻿using System.Windows;

using ICommand = System.Windows.Input.ICommand;
using ListView = System.Windows.Controls.ListView;

namespace OptimisationManagerExtention.ViewExtention
{
    /// <summary>
    /// Класс расширений для ListView который переводит события в команды (ICommand)
    /// класс помечен ключевым словом partial - т.е. его реализация разбита на несколько файлов.
    /// 
    /// Конкретно в данном классе - реализован перевод события ListView.SelectionChangedEvent 
    /// в комманду типа ICommand
    /// </summary>
    partial class ListViewExtention
    {

        #region Command
        /// <summary>
        /// Зависимое свойство - содержащее в себе ссылку на коллбек команды
        /// Свойство задается через View в XAML разметке проекта
        /// </summary>
        public static readonly DependencyProperty SelectionChangedCommandProperty =
            DependencyProperty.RegisterAttached("SelectionChangedCommand",
                typeof(ICommand), typeof(ListViewExtention),
                new PropertyMetadata(SelectionChangedCommandPropertyCallback));

        /// <summary>
        /// Setter для SelectionChangedCommandProperty
        /// </summary>
        /// <param name="obj">Элемент управления</param>
        /// <param name="value">Значение с которым осуществляется связка</param>
        public static void SetSelectionChangedCommand(UIElement obj, ICommand value)
        {
            obj.SetValue(SelectionChangedCommandProperty, value);
        }
        /// <summary>
        /// Geter для SelectionChangedCommandProperty
        /// </summary>
        /// <param name="obj">Элемент управления</param>
        /// <returns>ссылку на сохраненную команду типа ICommand</returns>
        public static ICommand GetSelectionChangedCommand(UIElement obj)
        {
            return (ICommand)obj.GetValue(SelectionChangedCommandProperty);
        }
        /// <summary>
        /// Коллбек вызываемый после задания свойства SelectionChangedCommandProperty
        /// </summary>
        /// <param name="obj">Элемент управления для которого задается свойство</param>
        /// <param name="args">события предшествующие вызову коллбека</param>
        private static void SelectionChangedCommandPropertyCallback(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            if (obj is ListView lw)
            {
                if (args.OldValue != null)
                    lw.SelectionChanged -= Lw_SelectionChanged;

                if (args.NewValue != null)
                    lw.SelectionChanged += Lw_SelectionChanged;
            }
        }
        /// <summary>
        /// Коллбек события которое переводится в тип ICommand
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void Lw_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (sender is UIElement element)
            {
                object param = GetSelectionChangedCommandParameter(element);
                ICommand cmd = GetSelectionChangedCommand(element);
                if (cmd.CanExecute(param))
                    cmd.Execute(param);
            }
        }
        #endregion

        #region CommandParameter
        /// <summary>
        /// Зависимое свойство - содержащее в себе ссылку на параметры передаваемые в коллбек типа ICommand
        /// Свойство задается через View в XAML разметке проекта
        /// </summary>
        public static readonly DependencyProperty SelectionChangedCommandParameterProperty =
            DependencyProperty.RegisterAttached("SelectionChangedCommandParameter",
                typeof(object), typeof(ListViewExtention));
        /// <summary>
        /// Setter для SelectionChangedCommandParameterProperty
        /// </summary>
        /// <param name="obj">Элемент управления</param>
        /// <param name="value">Значение с которым осуществляется связка</param>
        public static void SetSelectionChangedCommandParameter(UIElement obj, object value)
        {
            obj.SetValue(SelectionChangedCommandParameterProperty, value);
        }
        /// <summary>
        /// Geter для SelectionChangedCommandParameterProperty
        /// </summary>
        /// <param name="obj">Элемент управления</param>
        /// <returns>переданный параметр</returns>
        public static object GetSelectionChangedCommandParameter(UIElement obj)
        {
            return obj.GetValue(SelectionChangedCommandParameterProperty);
        }
        #endregion

    }
}
