﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Input;
using OptimisationManagerExtention.Model.Terminal;
using OptimisationManagerExtention.Model.FileReaders;
using OptimisationManagerExtention.Model;
using OptimisationManagerExtention.ViewExtention;
using System.Collections.ObjectModel;

using DataTable = System.Data.DataTable;
using System.Runtime.CompilerServices;

[assembly: InternalsVisibleTo("Test")]
[assembly: InternalsVisibleTo("OptimisationManagerExtentionTests")]
namespace OptimisationManagerExtention.ViewModel
{
    /// <summary>
    /// View Model
    /// </summary>
    class ExtentionGUI_VM : INotifyPropertyChanged
    {
        /// <summary>
        /// Model instance
        /// </summary>
        private readonly IExtentionGUI_M model = ModelCreator.Model;

        /// <summary>
        /// Конструктор
        /// </summary>
        public ExtentionGUI_VM()
        {
            // подписка на события из модели
            model.PropertyChanged += Model_PropertyChanged;
            model.TerminalsID.CollectionChanged += TerminalsID_CollectionChanged;
            //model.TerminalsAfterOptimisation.CollectionChanged += TerminalsAfterOptimisation_CollectionChanged;
            //model.BotsAfterOptimisation.CollectionChanged += BotsAfterOptimisation_CollectionChanged;

            // Коллбеки на команды
            AddTerminal = new RelayCommand(AddNewTerminal, CheckBeforeAddTerminal);
            Start = new RelayCommand(StartCallback, CheckBeforeStart);
            StartTestFromOptimisationResults = new RelayCommand(StartTestFromOptimisationResultsAction);
            UpdateOptimisationReport = new RelayCommand(UpdateReportsData);

            ExecutionList = new ComboBoxItems<string>
            {
                ItemSource = new List<string>(Enum.GetNames(typeof(ENUM_ExecutionDelay)).Select(x => x.Replace("_", " ")))
            };
            ExecutionList.Action += CB_Action;
            ModelList = new ComboBoxItems<string>
            {
                ItemSource = new List<string>(Enum.GetNames(typeof(ENUM_Model)).Select(x => x.Replace("_", " ")))
            };
            ModelList.Action += CB_Action;
            OptimisationCriteriaList = new ComboBoxItems<string>
            {
                ItemSource = new List<string>(Enum.GetNames(typeof(ENUM_OptimisationCriteria)).Select(x => x.Replace("__", " * ")).Select(x => x.Replace("_", " ")))
            };
            OptimisationCriteriaList.Action += CB_Action;
            Deposit = new ComboBoxItems<int>
            {
                ItemSource = new List<int>
                {
                    3000,
                    5000,
                    10000,
                    25000,
                    50000,
                    100000
                }
            };
            Deposit.Action += CB_Action;
            CurrencyList = new ComboBoxItems<string>
            {
                ItemSource = new List<string>
                {
                    "RUB",
                    "USD",
                    "EUR",
                    "GBP",
                    "CHF"
                }
            };
            CurrencyList.Action += CB_Action;
            LaverageList = new ComboBoxItems<string>
            {
                ItemSource = new List<string>
                {
                    "1:1",
                    "1:5",
                    "1:50",
                    "1:100",
                    "1:200",
                    "1:500"
                }
            };
            LaverageList.Action += CB_Action;
        }

        /// <summary>
        /// Деструктор
        /// </summary>
        ~ExtentionGUI_VM()
        {
            // Отписка от событий модели
            model.PropertyChanged -= Model_PropertyChanged;
            model.TerminalsID.CollectionChanged -= TerminalsID_CollectionChanged;
            //model.TerminalsAfterOptimisation.CollectionChanged -= TerminalsAfterOptimisation_CollectionChanged;
            //model.BotsAfterOptimisation.CollectionChanged -= BotsAfterOptimisation_CollectionChanged;
            ExecutionList.Action -= CB_Action;
            ModelList.Action -= CB_Action;
            OptimisationCriteriaList.Action -= CB_Action;
            Deposit.Action -= CB_Action;
            CurrencyList.Action -= CB_Action;
            LaverageList.Action -= CB_Action;
        }

        /// <summary>
        /// Событияе изменения какого либо из свойств ViewModel 
        /// и его обработчики
        /// </summary>
        #region PropertyChanged Event
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// Обработчик события PropertyChanged
        /// </summary>
        /// <param name="propertyName">Имя обновляемой переменной</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        /// <summary>
        /// Обработчик события PropertyChanged из модели.
        /// </summary>
        /// <param name="sender">класс отправитель (в данном случае - модель)</param>
        /// <param name="e">тип произошедшего события</param>
        private void Model_PropertyChanged(object sender, PropertyChangedEventArgs e) => OnPropertyChanged(e.PropertyName);
        private void TerminalsID_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SelectedTerminal = 0;
            OnPropertyChanged("SelectedTerminal");
        }
        //private void BotsAfterOptimisation_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        //{
        //    BotsAfterOptimisation_Selected = 0;
        //    OnPropertyChanged("BotssAfterOptimisation_Selected");
        //}
        //private void TerminalsAfterOptimisation_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        //{
        //    TerminalsAfterOptimisation_Selected = 0;
        //    OnPropertyChanged("TerminalsAfterOptimisation_Selected");
        //}
        #endregion

        #region Settings tab

        #region TerminalPaths & SelectedTerminal
        public bool IsTerminalsLVEnabled => model.IsTerminalsLVEnabled;

        /// <summary>
        /// Список роботов - данные для ComboBox со списком роботов
        /// Ссылается на поле "TerminalPaths" из модели
        /// </summary>
        public ObservableCollection<string> TerminalsID => model.TerminalsID;
        /// <summary>
        /// Индекс выбранного терминалла
        /// </summary>
        public int SelectedTerminal { get; set; } = 0;
        #endregion

        #region Add new terminal
        /// <summary>
        /// Обработчик команды добавления нового терминалла в таблицу терминалов
        /// </summary>
        public ICommand AddTerminal { get; }
        /// <summary>
        /// Проверка перед добавлением терминалла
        /// </summary>
        /// <param name="obj">переданный параметр из View</param>
        /// <returns>результат проверки возможности выполнения операции</returns>
        private bool CheckBeforeAddTerminal(object obj)
        {
            return TerminalsID.Count > 0;
        }
        /// <summary>
        /// Добавление нового терминалла в список терминалов
        /// Так как процедурра длительная - то она запускается асинхронно
        /// </summary>
        /// <param name="obj">переданный из View параметр</param>
        private void AddNewTerminal(object obj)
        {
            // Получаем индекс выбранного терминалла в локальную переменную на случай если он будет изменен за время работы метода
            // и проверяем существует ли список терминаллов
            int terminalIndex = SelectedTerminal;
            if (terminalIndex == -1 ||
                TerminalsID.Count == 0 ||
                !model.IsEnableToAddNewTerminal())
            {
                return;
            }

            // Получаем путь к эксперту
            string Terminal = TerminalsID[terminalIndex];

            if (SelectedTerminalsForOptimisation.Any(x => x.TerminalID == Terminal))
            {
                System.Windows.MessageBox.Show("Can`t add this terminal because it isalready added!");
                return;
            }

            List<string> botList = null;//, assetList = null;
            uint? _Login = null;

            // Асинхронно запрашиваем у модели список роботов, список активов, логин выбранного терминалла
            botList = model.GetBotNamesList(terminalIndex);
            //assetList = model.GetAssetNames(terminalIndex);
            _Login = model.GetCurrentLogin(terminalIndex);

            // Если существует список роботов и активов - то добавляем новый терминалл в коллекцию
            if (botList != null /*&& assetList != null*/ &&
                botList.Count > 0/* && assetList.Count > 0*/)
            {
                SelectedTerminalsForOptimisation.Add
                    (
                        new TerminalAndBotItem(botList, /*assetList,*/ Terminal, FillInBotParams, DeleteSelectedTerminal)
                        {
                            Login = _Login
                        }
                    );

                selectedTerminalIndex = SelectedTerminalsForOptimisation.Count - 1;
            }
            else
                System.Windows.MessageBox.Show("Can`t add new terminal becouse there are no experts in terminal");
        }
        #endregion

        #region SelectedTerminalsForOptimisation && SelectedTerminalIndex (first LV params)
        /// <summary>
        /// Список терминаллов выбранных для оптимизации отображаемый в таблице терминаллов
        /// </summary>
        public ObservableCollection<TerminalAndBotItem> SelectedTerminalsForOptimisation { get; private set; } =
            new ObservableCollection<TerminalAndBotItem>();
        /// <summary>
        /// Индекс выбранной строки
        /// </summary>
        private int selectedTerminalIndex = 0;
        public int SelectedTerminalIndex
        {
            get { return selectedTerminalIndex; }
            set
            {
                // Присваиваем значение нововыбранного индекса
                selectedTerminalIndex = value;

                //((RelayCommand)Start).OnCanExecuteChanged();

                // Заполняем список параметров выбранного в текущей строке робота
                if (value == -1)
                {
                    return;
                }
                TerminalAndBotItem terminal_item = SelectedTerminalsForOptimisation[value];
                if (terminal_item.Experts.Count > 0)
                {
                    FillInBotParams(terminal_item.Experts[terminal_item.SelectedExpert],
                        terminal_item.TerminalID);
                }
            }
        }
        #endregion
        /// <summary>
        /// Список параметров робота как в таблице перед началом оптимизации
        /// </summary>
        public List<ParamsItem> BotParams => model.BotParams;

        #region Optimisation and Test settings

        /// <summary>
        /// Логин который робот видит во время теста (нужно если есть ограничения по логину)
        /// </summary>
        private uint? _tertLogin;
        public uint? TestLogin
        {
            get => _tertLogin;
            set
            {
                _tertLogin = value;

                OnPropertyChanged("TestLogin");
                CB_Action(GetSetActionType.Set_Index);
            }
        }
        /// <summary>
        /// Задержка исполнения ордеров
        /// </summary>
        public ComboBoxItems<string> ExecutionList { get; }
        /// <summary>
        /// Тип используемых котировок (каждый  тик OHLC 1M ...)
        /// </summary>
        public ComboBoxItems<string> ModelList { get; }
        /// <summary>
        /// Критерий оптимизации
        /// </summary>
        public ComboBoxItems<string> OptimisationCriteriaList { get; }
        /// <summary>
        /// Депозит
        /// </summary>
        public ComboBoxItems<int> Deposit { get; }
        /// <summary>
        /// Валюта рассчета прибыли
        /// </summary>
        public ComboBoxItems<string> CurrencyList { get; }
        /// <summary>
        /// Кредитное плечо
        /// </summary>
        public ComboBoxItems<string> LaverageList { get; }
        /// <summary>
        /// Дата начала форвардного теста
        /// </summary>
        private DateTime _DTForvard = DateTime.Now;
        public DateTime ForvardDate
        {
            get => _DTForvard;
            set
            {
                _DTForvard = value;

                OnPropertyChanged("ForvardDate");
                CB_Action(GetSetActionType.Set_Index);
            }
        }
        /// <summary>
        /// Признак запуска тестера в графическом режиме
        /// </summary>
        private bool _isVisualMode = false;
        /// <summary>
        /// Признак запуска темтера в визуальном режиме
        /// </summary>
        public bool IsVisual
        {
            get => _isVisualMode;
            set
            {
                _isVisualMode = value;

                OnPropertyChanged("IsVisual");
                CB_Action(GetSetActionType.Set_Index);
            }
        }
        /// <summary>
        /// скрытая переменная хрянящая значение флага IsSaveInModel
        /// </summary>
        private bool isSaveInModel = true;
        /// <summary>
        /// Разделяемый рессурс для асинхронного доступа к свойству IsSaveInModel
        /// </summary>
        private readonly object SaveModel_locker = new object();
        /// <summary>
        /// Флаг если True - то при изменении параметров для тестера они будут сохраняться
        /// </summary>
        private bool IsSaveInModel
        {
            get
            {
                lock (SaveModel_locker)
                    return isSaveInModel;
            }
            set
            {
                lock (SaveModel_locker)
                    isSaveInModel = value;
            }
        }
        /// <summary>
        /// Коллбек сохраняющий изменения в параметрах тестера
        /// </summary>
        /// <param name="actionType"></param>
        private void CB_Action(GetSetActionType actionType)
        {
            if (actionType == GetSetActionType.Set_Index && IsSaveInModel)
            {
                model.UpdateTerminalOptimisationsParams(new OptimisationInputData
                {
                    Login = TestLogin,
                    IsVisual = IsVisual,
                    ForvardDate = ForvardDate,
                    CurrencyIndex = CurrencyList.SelectedIndex,
                    DepositIndex = Deposit.SelectedIndex,
                    ExecutionDelayIndex = ExecutionList.SelectedIndex,
                    LaverageIndex = LaverageList.SelectedIndex,
                    ModelIndex = ModelList.SelectedIndex,
                    OptimisationCriteriaIndex = OptimisationCriteriaList.SelectedIndex,
                    Deposit = Deposit.ItemSource[Deposit.SelectedIndex],
                    Currency = CurrencyList.ItemSource[CurrencyList.SelectedIndex],
                    Laverage = LaverageList.ItemSource[LaverageList.SelectedIndex]
                });
            }
        }
        #endregion

        #region Operation status
        /// <summary>
        /// Статус выполняемой операции
        /// </summary>
        public string Status => model.Status;
        /// <summary>
        /// Значение прогрессбара
        /// </summary>
        public double PB_Value
        {
            get { return model.PB_Value; }
            set { model.PB_Value.SetVarSilently(value); }
        }
        #endregion

        #region Start Optimisation/test 
        /// <summary>
        /// Комманда запуска оптимизации/теста
        /// </summary>
        public ICommand Start { get; }
        /// <summary>
        /// Метод проверки перед оптимизацией / тестом
        /// </summary>
        /// <param name="obj"></param>
        /// <returns>результат проверки</returns>
        private bool CheckBeforeStart(object obj)
        {
            return SelectedTerminalsForOptimisation.Count > 0;
        }
        /// <summary>
        /// Пуск оптимизации / теста
        /// </summary>
        /// <param name="obj"></param>
        private void StartCallback(object obj)
        {
            model.StartOptimisationOrTest(SelectedTerminalsForOptimisation.ToList());
        }
        #endregion

        #endregion

        #region Optimisation Result

        /// <summary>
        /// Таблица с результатами исторической оптимизацией
        /// </summary>
        public DataTable HistoryOptimisationResults => model.HistoryOptimisationResults;
        /// <summary>
        /// Таблица с резальтатами форвардной оптимизацией
        /// </summary>
        public DataTable ForvardOptimisationResults => model.ForvardOptimisationResults;
        /// <summary>
        /// Наблюдаемая коллекция со списком колонок оптимизации
        /// </summary>
        public ObservableCollection<ColumnDescriptor> OptimisationResultsColumnHeadders =>
               model.OptimisationResultsColumnHeadders;

        #region Start test from optimisation results
        /// <summary>
        /// Запуск теста для выбранного оптимизационныго прозода
        /// </summary>
        public ICommand StartTestFromOptimisationResults { get; }
        /// <summary>
        /// Метод запускающий тест по двойному клику
        /// </summary>
        /// <param name="type"></param>
        private void StartTestFromOptimisationResultsAction(object type)
        {
            ENUM_TableType tableType = (string)type == "History" ?
                ENUM_TableType.History : ENUM_TableType.Forvard;
            int ind = tableType == ENUM_TableType.History ?
                SelectedHistoryOptimisationRow : SelectedForvardOptimisationRow;

            model.StartTest(tableType, ind);
        }
        #endregion

        /// <summary>
        /// индекс выбранной строки из таблицы исторических оптимизаций
        /// </summary>
        public int SelectedHistoryOptimisationRow { get; set; } = 0;
        /// <summary>
        /// Индекс выбранной строки форвардной оптимизации
        /// </summary>
        public int SelectedForvardOptimisationRow { get; set; } = 0;

        #region UpdateOptimisationReport

        #region TerminalsAfterOptimisation
        public ObservableCollection<string> TerminalsAfterOptimisation => model.TerminalsAfterOptimisation;
        public int TerminalsAfterOptimisation_Selected
        {
            get => model.TerminalsAfterOptimisation_Selected;
            set
            {
                model.TerminalsAfterOptimisation_Selected.SetVarSilently(value);
                if (value > -1)
                    model.SelectNewBotsAfterOptimisation_forNewTerminal();
            }
        }
        
        public ObservableCollection<string> BotsAfterOptimisation => model.BotsAfterOptimisation;
        public int BotsAfterOptimisation_Selected
        {
            get => model.BotsAfterOptimisation_Selected;
            set => model.BotsAfterOptimisation_Selected.SetVarSilently(value);
        }
        #endregion

        public ICommand UpdateOptimisationReport { get; }

        //private bool CheckIsReportsExists(object o)
        //{
        //    return model.IsEnableLoadOptimisationReport();
        //}
        private void UpdateReportsData(object o)
        {
            model.LoadOptimisations();
        }
        #endregion
        #endregion

        #region Methods - Callbacks for TerminalAndBotItem class

        private readonly object botParams_locker = new object();
        /// <summary>
        /// Получение и заполнение параметров робота
        /// </summary>
        /// <param name="fullExpertName">Полное имя эксперта относительно папки ~/Experts</param>
        /// <param name="Terminal">ID терминалла</param>
        private async void FillInBotParams(string fullExpertName, string Terminal)
        {
            await System.Threading.Tasks.Task.Run(() =>
            {
                lock (botParams_locker)
                {
                    model.LoadBotParams(fullExpertName, Terminal, out OptimisationInputData? optimisationData);
                    if (!optimisationData.HasValue)
                        return;

                    IsSaveInModel = false;
                    TestLogin = optimisationData.Value.Login;
                    IsVisual = optimisationData.Value.IsVisual;
                    ForvardDate = optimisationData.Value.ForvardDate;
                    CurrencyList.SelectedIndex = optimisationData.Value.CurrencyIndex;
                    Deposit.SelectedIndex = optimisationData.Value.DepositIndex;
                    ExecutionList.SelectedIndex = optimisationData.Value.ExecutionDelayIndex;
                    LaverageList.SelectedIndex = optimisationData.Value.LaverageIndex;
                    ModelList.SelectedIndex = optimisationData.Value.ModelIndex;
                    OptimisationCriteriaList.SelectedIndex = optimisationData.Value.OptimisationCriteriaIndex;
                    IsSaveInModel = true;
                }
            });


            OnPropertyChanged("BotParams");
        }
        /// <summary>
        /// Метод вызывается как коллбек на нажатие кнопки "Delete" в списке терминаллов
        /// </summary>
        /// <param name="item">Переданный на удаление из списка терминалл</param>
        private void DeleteSelectedTerminal(TerminalAndBotItem item)
        {
            if (model.RemoveBotParams(item.TerminalID))
            {
                SelectedTerminalsForOptimisation.Remove(item);

                if (SelectedTerminalsForOptimisation.Count > 0)
                    SelectedTerminalIndex = SelectedTerminalsForOptimisation.Count - 1;
                else
                    OnPropertyChanged("BotParams");
                //((RelayCommand)StartTestFromOptimisationResults).OnCanExecuteChanged();
            }
        }
        #endregion
    }

    #region Accessory Objects
    /// <summary>
    /// Класс - обертка для данных ComboBox списков
    /// </summary>
    /// <typeparam name="T">Тип данных хранимых к ComboBox</typeparam>
    class ComboBoxItems<T> : INotifyPropertyChanged
    {
        /// <summary>
        /// Коллекция эллементов списка
        /// </summary>
        private List<T> items;
        public List<T> ItemSource
        {
            get
            {
                OnAction(GetSetActionType.Get_Value);
                return items;
            }
            set
            {
                items = value;
                OnAction(GetSetActionType.Set_Value);
            }
        }
        /// <summary>
        /// Выбранный индекс в списке
        /// </summary>
        int selectedIndex = 0;
        public int SelectedIndex
        {
            get
            {
                OnAction(GetSetActionType.Get_Index);
                return selectedIndex;
            }
            set
            {
                selectedIndex = value;
                OnAction(GetSetActionType.Set_Index);
            }
        }

        public event Action<GetSetActionType> Action;
        public event PropertyChangedEventHandler PropertyChanged;

        private void OnAction(GetSetActionType type)
        {
            switch (type)
            {
                case GetSetActionType.Set_Value:
                    PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("ItemSource"));
                    break;
                case GetSetActionType.Set_Index:
                    PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("SelectedIndex"));
                    break;
            }
            Action?.Invoke(type);
        }
    }
    enum GetSetActionType
    {
        Get_Value,
        Set_Value,
        Get_Index,
        Set_Index
    }


    /// <summary>
    /// Класс описывающий строки в таблице терминалов выбранных для оптимизации / тестов
    /// </summary>
    class TerminalAndBotItem
    {
        /// <summary>
        /// Конструктор класса 
        /// </summary>
        /// <param name="botList">С
        /// писок наименований полных путей до роботов 
        /// относительно папки ~/Experts терминалла
        /// </param>
        /// <param name="pathToTerminal">
        /// Путь к стартовому файлу терминалла
        /// </param>
        /// <param name="FillInBotParams">
        /// Метод мереданный через делегат.
        /// Данный тетод заполняет список входных параметров робота 
        /// а так же настроек его оптимизации
        /// </param>
        /// <param name="PathToExpertFolder">
        /// Путь к папке ~/Experts выбранного терминалла
        /// </param>
        /// <param name="DeleteCommand">
        /// Метод переданный через делегат - удаляющий данную строку из таблицы
        /// </param>
        public TerminalAndBotItem(List<string> botList,
            //List<string> Assets,
            string TerminalID,
            Action<string, string> FillInBotParams,
            Action<TerminalAndBotItem> DeleteCommand)
        {
            // Заполняем поля делегатов
            #region Delegates
            this.FillInBotParams = FillInBotParams;
            this.DeleteCommand = new RelayCommand((object o) => DeleteCommand(this));
            #endregion

            // Сохраняем пути до папок терминалла
            this.TerminalID = TerminalID;

            // Создаем клвссы описывающий данные ComboBox и заполняем его данными
            /* this.Assets = new ComboBoxItems<string>
             {
                 ItemSource = new List<string>(Assets),
                 SelectedIndex = 0
             };*/
            TF = new ComboBoxItems<string>
            {
                ItemSource = new List<string>(Enum.GetNames(typeof(ENUM_Timeframes))),
                SelectedIndex = 0
            };
            OptimisationMode = new ComboBoxItems<string>
            {
                ItemSource = new List<string>(Enum.GetNames(typeof(ENUM_OptimisationMode)).Select(x => x.Replace("_", " "))),
                SelectedIndex = 0
            };

            // Заполняем поля для ComboBox с выбором экспертов.
            // не исспользуем класс ComboBoxItems потому что нам понадобится назначить
            // Коллбек для события выбора нового робота
            #region Expert ComboBox
            Experts = new List<string>(botList);
            SelectedExpert = 0;
            #endregion
        }

        #region Delegates
        /// <summary>
        /// Поле с делегатом обновления параметров выбранного робота
        /// </summary>
        private readonly Action<string, string> FillInBotParams;
        /// <summary>
        /// Коллбек для команды удаления терминалла из списка (кнопка Delete в таблице)
        /// </summary>
        public ICommand DeleteCommand { get; }
        #endregion

        /// <summary>
        /// Путь к папке со стартовым файлом запуска терминалла (*.exe)
        /// </summary>
        public string TerminalID { get; }

        #region Expert ComboBox
        /// <summary>
        /// Список доступных экспертов
        /// </summary>
        public List<string> Experts { get; }
        /// <summary>
        /// индекс выбранного эксперта
        /// </summary>
        private int selectedExpert;
        /// <summary>
        /// Property для индекса выбранного эксперта
        /// </summary>
        public int SelectedExpert
        {
            get { return selectedExpert; }
            set
            {
                selectedExpert = value;
                // Запускаем коллбек загружки параметров для выбранного эксперта 
                if (Experts.Count > 0)
                    FillInBotParams(Experts[selectedExpert], TerminalID);
            }
        }
        #endregion
        /// <summary>
        /// Эллементы ComboBox со списком доступных активов
        /// </summary>
       // public ComboBoxItems<string> Assets { get; }
        public string AssetName { get; set; }
        /// <summary>
        /// Эллементы ComboBox со списком доступных таймфреймов
        /// </summary>
        public ComboBoxItems<string> TF { get; }
        /// <summary>
        /// Эллементы ComboBox со списком доступных способов оптимизации
        /// </summary>
        public ComboBoxItems<string> OptimisationMode { get; }

        /// <summary>
        /// Дата начала тестирования/оптимизирования
        /// </summary>
        public DateTime From { get; set; } = DateTime.Now;
        /// <summary>
        /// Дата окончания тестирования/оптимизирования 
        /// </summary>
        public DateTime Till { get; set; } = DateTime.Now;
        /// <summary>
        /// Логин исспользуемый для входа в терминалл
        /// </summary>
        public uint? Login { get; set; }
        /// <summary>
        /// Пароль исспользуемый для входа в терминалл
        /// </summary>
        public string Pass { get; set; }
        /// <summary>
        /// Пароль для сертефиката (если исспользуется расширенная аутентификация)
        /// </summary>
        public string CertPass { get; set; }
        /// <summary>
        /// Сервер исспользуемый для подключения терминалла
        /// </summary>
        public string Server { get; set; }

        /// <summary>
        /// Созданные для удобства дальнейшего программирования методы 
        /// которые автоматически конвертируют в enum
        /// выбранные пользователем параметры оптимизации / теста
        /// </summary>
        #region String back To Enam
        public ENUM_Timeframes GetTF =>
            (ENUM_Timeframes)Enum.Parse(typeof(ENUM_Timeframes), TF.ItemSource[TF.SelectedIndex]);
        public ENUM_OptimisationMode GetOptimisationMode =>
            (ENUM_OptimisationMode)Enum.Parse(typeof(ENUM_OptimisationMode), OptimisationMode.ItemSource[OptimisationMode.SelectedIndex].Replace(" ", "_"));
        #endregion
    }
    #endregion
}
