﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Linq;
using OptimisationManagerExtention;
using OptimisationManagerExtention.Model;
using OptimisationManagerExtention.Model.FileReaders;
using OptimisationManagerExtention.ViewExtention;
using OptimisationManagerExtention.ViewModel;
using OptimisationManagerExtention.Model.DirectoryManagers;

namespace Test
{
    class Program
    {
        static void Main(string[] args)
        {
            ModelCreator.SetModel(new MyTestModel());

            MQL5Connector.Instance("1B9501BF48F2354A4685940A72752910");
        }
    }

    class MyTestModel : IExtentionGUI_M
    {
        public MyTestModel()
        {
            Status = new VarKeeper<string>("Status", "Test Model");
            Status.PropertyChanged += OnPropertyChanged;
            PB_Value = new VarKeeper<double>("PB_Value", 50);
            PB_Value.PropertyChanged += OnPropertyChanged;
            TerminalsID = new ObservableCollection<string>(terminalDirectory.Terminals.Select(x => x.Name));
            selectedTerminal = new VarKeeper<int>("TerminalsAfterOptimisation_Selected", 0);
            selectedTerminal.PropertyChanged += OnPropertyChanged;
            selectedBots = new VarKeeper<int>("BotsAfterOptimisation_Selected", 0);
            selectedBots.PropertyChanged += OnPropertyChanged;
        }

        ~MyTestModel()
        {
            Status.PropertyChanged -= OnPropertyChanged;
            PB_Value.PropertyChanged -= OnPropertyChanged;
            selectedTerminal.PropertyChanged -= OnPropertyChanged;
            selectedBots.PropertyChanged -= OnPropertyChanged;
        }

        private readonly TerminalDirectory terminalDirectory = new TerminalDirectory();

        public bool IsTerminalsLVEnabled { get; private set; } = true;

        public List<ParamsItem> BotParams { get; private set; } = new List<ParamsItem>();

        public VarKeeper<string> Status { get; private set; }

        public VarKeeper<double> PB_Value { get; private set; }

        public ObservableCollection<string> TerminalsID { get; }

        public DataTable HistoryOptimisationResults => new DataTable();

        public DataTable ForvardOptimisationResults => new DataTable();

        public ObservableCollection<ColumnDescriptor> OptimisationResultsColumnHeadders => new ObservableCollection<ColumnDescriptor>()
        {
            new ColumnDescriptor
            {
                DisplayMember = "Col_1",
                HeaderText = "Col 1"
            },
            new ColumnDescriptor
            {
                DisplayMember = "Col_2",
                HeaderText = "Col 2"
            }
        };

        public ObservableCollection<string> TerminalsAfterOptimisation => new ObservableCollection<string>();

        private VarKeeper<int> selectedTerminal;
        public VarKeeper<int> TerminalsAfterOptimisation_Selected { get => selectedTerminal; set => selectedTerminal.SetVar(value); }

        public ObservableCollection<string> BotsAfterOptimisation => new ObservableCollection<string>();

        private VarKeeper<int> selectedBots;
        public VarKeeper<int> BotsAfterOptimisation_Selected { get => selectedBots; set => selectedBots.SetVar(value); }

        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public List<string> GetBotNamesList(int terminalIndex)
        {
            return new List<string>
            {
                "Bot 1",
                "Bot 2",
                "Bot 3"
            };
        }

        public uint? GetCurrentLogin(int terminalIndex)
        {
            Random rnd = new Random();

            return (uint)rnd.Next(10000, 99999);
        }

        public bool IsEnableToAddNewTerminal()
        {
            return true;
        }

        public void LoadBotParams(string fullExpertName, string TerminalID, out OptimisationInputData? optimisationInputData)
        {
            optimisationInputData = new OptimisationInputData();
            Console.WriteLine("LoadBotParams");
        }

        public void LoadOptimisations()
        {
            Console.WriteLine("LoadOptimisations");
        }

        public bool RemoveBotParams(string TerminalID)
        {
            Console.WriteLine("RemoveBotParams");
            return true;
        }

        public void SelectNewBotsAfterOptimisation_forNewTerminal()
        {
            Console.WriteLine("SelectNewBotsAfterOptimisation_forNewTerminal");
        }

        public void StartOptimisationOrTest(List<TerminalAndBotItem> SelectedTerminals)
        {
            Console.WriteLine("StartOptimisationOrTest");
        }

        public void StartTest(ENUM_TableType TableType, int rowIndex)
        {
            Console.WriteLine("StartTest");
        }

        public void UpdateTerminalOptimisationsParams(OptimisationInputData optimisationInputData)
        {
            Console.WriteLine("UpdateTerminalOptimisationsParams");
        }
    }

}
